@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Unevaluated;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Unevaluated vocabulary
 
-our $VERSION = '0.626';
+our $VERSION = '0.627';
 
 use 5.020;
 use Moo;
@@ -48,8 +48,6 @@ sub _eval_keyword_unevaluatedItems ($class, $data, $schema, $state) {
   # these should never happen
   die '"unevaluatedItems" keyword present, but annotation collection is disabled'
     if not $state->{collect_annotations};
-  die '"unevaluatedItems" keyword present, but short_circuit is enabled: results unreliable'
-    if $state->{short_circuit};
 
   my @annotations = local_annotations($state);
 
@@ -109,8 +107,6 @@ sub _eval_keyword_unevaluatedProperties ($class, $data, $schema, $state) {
   # these should never happen
   die '"unevaluatedProperties" keyword present, but annotation collection is disabled'
     if not $state->{collect_annotations};
-  die '"unevaluatedProperties" keyword present, but short_circuit is enabled: results unreliable'
-    if $state->{short_circuit};
 
   my @evaluated_properties = map {
     my $keyword = $_->{keyword};
@@ -161,7 +157,7 @@ JSON::Schema::Modern::Vocabulary::Unevaluated - Implementation of the JSON Schem
 
 =head1 VERSION
 
-version 0.626
+version 0.627
 
 =head1 DESCRIPTION
 
